@@ -13,9 +13,10 @@
 import queue
 import threading
 import time
+import types
 
 
-class FairSemaphore(object):
+class FairSemaphore:
     """Semaphore class that notifies in order of request.
 
     We cannot use a normal Semaphore because it doesn't give any ordering,
@@ -29,17 +30,17 @@ class FairSemaphore(object):
         semaphore.
     """
 
-    def __init__(self, concurrency, rate_delay):
+    def __init__(self, concurrency: int | None, rate_delay: float):
         self._lock = threading.Lock()
         self._concurrency = concurrency
         if concurrency:
             self._count = 0
-            self._queue = queue.Queue()
+            self._queue: queue.Queue[threading.Event] = queue.Queue()
 
         self._rate_delay = rate_delay
         self._rate_last_ts = time.time()
 
-    def __enter__(self):
+    def __enter__(self) -> None:
         """Aquire a semaphore."""
         # If concurrency is None, everyone is free to immediately execute.
         if not self._concurrency:
@@ -51,13 +52,17 @@ class FairSemaphore(object):
             execution_time = self._get_ticket()
         return self._wait_for_execution(execution_time)
 
-    def _wait_for_execution(self, execution_time):
+    def _wait_for_execution(self, execution_time: float) -> None:
         """Wait until the pre-calculated time to run."""
         wait_time = execution_time - time.time()
         if wait_time > 0:
             time.sleep(wait_time)
 
-    def _get_ticket(self):
+    def _get_ticket(self) -> float:
+        # narrow types - we will only get here if self._concurrency is set
+        assert self._concurrency is not None  # nosec B101
+        assert self._count is not None  # nosec B101
+
         ticket = threading.Event()
         with self._lock:
             if self._count <= self._concurrency:
@@ -74,7 +79,7 @@ class FairSemaphore(object):
         with self._lock:
             return self._advance_timer()
 
-    def _advance_timer(self):
+    def _advance_timer(self) -> float:
         """Calculate the time when it's ok to run a command again.
 
         This runs inside of the mutex, serializing the calculation
@@ -85,7 +90,12 @@ class FairSemaphore(object):
         self._rate_last_ts = self._rate_last_ts + self._rate_delay
         return self._rate_last_ts
 
-    def __exit__(self, exc_type, exc_value, traceback):
+    def __exit__(
+        self,
+        exc_type: type[BaseException] | None,
+        exc_value: BaseException | None,
+        traceback: types.TracebackType | None,
+    ) -> None:
         """Release the semaphore."""
         # If concurrency is None, everyone is free to immediately execute
         if not self._concurrency:
