@@ -58,20 +58,20 @@ class _OidcBase(federation.FederationBaseAuth, metaclass=abc.ABCMeta):
         identity_provider: str,
         protocol: str,
         client_id: str,
-        client_secret: ty.Optional[str],
+        client_secret: str | None,
         access_token_type: str,
         scope: str,
-        access_token_endpoint: ty.Optional[str],
-        discovery_endpoint: ty.Optional[str],
+        access_token_endpoint: str | None,
+        discovery_endpoint: str | None,
         *,
-        trust_id: ty.Optional[str],
-        system_scope: ty.Optional[str],
-        domain_id: ty.Optional[str],
-        domain_name: ty.Optional[str],
-        project_id: ty.Optional[str],
-        project_name: ty.Optional[str],
-        project_domain_id: ty.Optional[str],
-        project_domain_name: ty.Optional[str],
+        trust_id: str | None,
+        system_scope: str | None,
+        domain_id: str | None,
+        domain_name: str | None,
+        project_id: str | None,
+        project_name: str | None,
+        project_domain_id: str | None,
+        project_domain_name: str | None,
         reauthenticate: bool,
         include_catalog: bool,
     ):
@@ -139,7 +139,7 @@ class _OidcBase(federation.FederationBaseAuth, metaclass=abc.ABCMeta):
         self.client_secret = client_secret
 
         self.discovery_endpoint = discovery_endpoint
-        self._discovery_document: ty.Dict[str, object] = {}
+        self._discovery_document: dict[str, object] = {}
         self.access_token_endpoint = access_token_endpoint
 
         self.access_token_type = access_token_type
@@ -147,7 +147,7 @@ class _OidcBase(federation.FederationBaseAuth, metaclass=abc.ABCMeta):
 
     def _get_discovery_document(
         self, session: ks_session.Session
-    ) -> ty.Dict[str, object]:
+    ) -> dict[str, object]:
         """Get the contents of the OpenID Connect Discovery Document.
 
         This method grabs the contents of the OpenID Connect Discovery Document
@@ -211,9 +211,7 @@ class _OidcBase(federation.FederationBaseAuth, metaclass=abc.ABCMeta):
             raise exceptions.OidcAccessTokenEndpointNotFound()
         return ty.cast(str, endpoint)
 
-    def _sanitize(
-        self, data: ty.Dict[str, ty.Optional[str]]
-    ) -> ty.Dict[str, ty.Optional[str]]:
+    def _sanitize(self, data: dict[str, str | None]) -> dict[str, str | None]:
         sanitized = copy.deepcopy(data)
         for key in sanitized:
             if any(s in key for s in SENSITIVE_KEYS):
@@ -221,9 +219,7 @@ class _OidcBase(federation.FederationBaseAuth, metaclass=abc.ABCMeta):
         return sanitized
 
     def _get_access_token(
-        self,
-        session: ks_session.Session,
-        payload: ty.Dict[str, ty.Optional[str]],
+        self, session: ks_session.Session, payload: dict[str, str | None]
     ) -> str:
         """Exchange a variety of user supplied values for an access token.
 
@@ -302,9 +298,8 @@ class _OidcBase(federation.FederationBaseAuth, metaclass=abc.ABCMeta):
         )
         return auth_response
 
-    # TODO(stephenfin): Deprecate and remove unused kwargs
     def get_unscoped_auth_ref(
-        self, session: ks_session.Session, **kwargs: ty.Any
+        self, session: ks_session.Session
     ) -> access.AccessInfoV3:
         """Authenticate with OpenID Connect and get back claims.
 
@@ -331,7 +326,7 @@ class _OidcBase(federation.FederationBaseAuth, metaclass=abc.ABCMeta):
         # First of all, check if the grant type is supported
         discovery = self._get_discovery_document(session)
         grant_types = ty.cast(
-            ty.Optional[ty.List[str]], discovery.get("grant_types_supported")
+            list[str] | None, discovery.get("grant_types_supported")
         )
         if (
             grant_types
@@ -357,7 +352,7 @@ class _OidcBase(federation.FederationBaseAuth, metaclass=abc.ABCMeta):
     @abc.abstractmethod
     def get_payload(
         self, session: ks_session.Session
-    ) -> ty.Dict[str, ty.Optional[str]]:
+    ) -> dict[str, str | None]:
         """Get the plugin specific payload for obtainin an access token.
 
         OpenID Connect supports different grant types. This method should
@@ -388,20 +383,20 @@ class OidcPassword(_OidcBase):
         client_secret: str,
         access_token_type: str = 'access_token',  # nosec B107
         scope: str = 'openid profile',
-        access_token_endpoint: ty.Optional[str] = None,
-        discovery_endpoint: ty.Optional[str] = None,
-        username: ty.Optional[str] = None,
-        password: ty.Optional[str] = None,
-        idp_otp_key: ty.Optional[str] = None,
+        access_token_endpoint: str | None = None,
+        discovery_endpoint: str | None = None,
+        username: str | None = None,
+        password: str | None = None,
+        idp_otp_key: str | None = None,
         *,
-        trust_id: ty.Optional[str] = None,
-        system_scope: ty.Optional[str] = None,
-        domain_id: ty.Optional[str] = None,
-        domain_name: ty.Optional[str] = None,
-        project_id: ty.Optional[str] = None,
-        project_name: ty.Optional[str] = None,
-        project_domain_id: ty.Optional[str] = None,
-        project_domain_name: ty.Optional[str] = None,
+        trust_id: str | None = None,
+        system_scope: str | None = None,
+        domain_id: str | None = None,
+        domain_name: str | None = None,
+        project_id: str | None = None,
+        project_name: str | None = None,
+        project_domain_id: str | None = None,
+        project_domain_name: str | None = None,
         reauthenticate: bool = True,
         include_catalog: bool = True,
     ):
@@ -440,7 +435,7 @@ class OidcPassword(_OidcBase):
 
     def get_payload(
         self, session: ks_session.Session
-    ) -> ty.Dict[str, ty.Optional[str]]:
+    ) -> dict[str, str | None]:
         """Get an authorization grant for the "password" grant type.
 
         :param session: a session object to send out HTTP requests.
@@ -453,7 +448,6 @@ class OidcPassword(_OidcBase):
             'username': self.username,
             'password': self.password,
             'scope': self.scope,
-            'client_id': self.client_id,
         }
 
         self.manage_otp_from_session_or_request_to_the_user(payload, session)
@@ -461,9 +455,7 @@ class OidcPassword(_OidcBase):
         return payload
 
     def manage_otp_from_session_or_request_to_the_user(
-        self,
-        payload: ty.Dict[str, ty.Optional[str]],
-        session: ks_session.Session,
+        self, payload: dict[str, str | None], session: ks_session.Session
     ) -> None:
         """Get the OTP code from the session or else request to the user.
 
@@ -510,17 +502,17 @@ class OidcClientCredentials(_OidcBase):
         client_secret: str,
         access_token_type: str = 'access_token',  # nosec B107
         scope: str = 'openid profile',
-        access_token_endpoint: ty.Optional[str] = None,
-        discovery_endpoint: ty.Optional[str] = None,
+        access_token_endpoint: str | None = None,
+        discovery_endpoint: str | None = None,
         *,
-        trust_id: ty.Optional[str] = None,
-        system_scope: ty.Optional[str] = None,
-        domain_id: ty.Optional[str] = None,
-        domain_name: ty.Optional[str] = None,
-        project_id: ty.Optional[str] = None,
-        project_name: ty.Optional[str] = None,
-        project_domain_id: ty.Optional[str] = None,
-        project_domain_name: ty.Optional[str] = None,
+        trust_id: str | None = None,
+        system_scope: str | None = None,
+        domain_id: str | None = None,
+        domain_name: str | None = None,
+        project_id: str | None = None,
+        project_name: str | None = None,
+        project_domain_id: str | None = None,
+        project_domain_name: str | None = None,
         reauthenticate: bool = True,
         include_catalog: bool = True,
     ):
@@ -556,7 +548,7 @@ class OidcClientCredentials(_OidcBase):
 
     def get_payload(
         self, session: ks_session.Session
-    ) -> ty.Dict[str, ty.Optional[str]]:
+    ) -> dict[str, str | None]:
         """Get an authorization grant for the client credentials grant type.
 
         :param session: a session object to send out HTTP requests.
@@ -565,7 +557,7 @@ class OidcClientCredentials(_OidcBase):
         :returns: a python dictionary containing the payload to be exchanged
         :rtype: dict
         """
-        payload: ty.Dict[str, ty.Optional[str]] = {'scope': self.scope}
+        payload: dict[str, str | None] = {'scope': self.scope}
         return payload
 
 
@@ -583,21 +575,21 @@ class OidcAuthorizationCode(_OidcBase):
         client_secret: str,
         access_token_type: str = 'access_token',  # nosec B107
         scope: str = 'openid profile',
-        access_token_endpoint: ty.Optional[str] = None,
-        discovery_endpoint: ty.Optional[str] = None,
-        code: ty.Optional[str] = None,
+        access_token_endpoint: str | None = None,
+        discovery_endpoint: str | None = None,
+        code: str | None = None,
         *,
-        trust_id: ty.Optional[str] = None,
-        system_scope: ty.Optional[str] = None,
-        domain_id: ty.Optional[str] = None,
-        domain_name: ty.Optional[str] = None,
-        project_id: ty.Optional[str] = None,
-        project_name: ty.Optional[str] = None,
-        project_domain_id: ty.Optional[str] = None,
-        project_domain_name: ty.Optional[str] = None,
+        trust_id: str | None = None,
+        system_scope: str | None = None,
+        domain_id: str | None = None,
+        domain_name: str | None = None,
+        project_id: str | None = None,
+        project_name: str | None = None,
+        project_domain_id: str | None = None,
+        project_domain_name: str | None = None,
         reauthenticate: bool = True,
         include_catalog: bool = True,
-        redirect_uri: ty.Optional[str] = None,
+        redirect_uri: str | None = None,
     ):
         """The OpenID Authorization Code plugin expects the following.
 
@@ -634,7 +626,7 @@ class OidcAuthorizationCode(_OidcBase):
 
     def get_payload(
         self, session: ks_session.Session
-    ) -> ty.Dict[str, ty.Optional[str]]:
+    ) -> dict[str, str | None]:
         """Get an authorization grant for the "authorization_code" grant type.
 
         :param session: a session object to send out HTTP requests.
@@ -660,18 +652,18 @@ class OidcAccessToken(_OidcBase):
         # sense with an access token
         access_token_type: str = 'access_token',  # nosec B107
         scope: str = 'openid profile',
-        access_token_endpoint: ty.Optional[str] = None,
-        discovery_endpoint: ty.Optional[str] = None,
-        access_token: ty.Optional[str] = None,
+        access_token_endpoint: str | None = None,
+        discovery_endpoint: str | None = None,
+        access_token: str | None = None,
         *,
-        trust_id: ty.Optional[str] = None,
-        system_scope: ty.Optional[str] = None,
-        domain_id: ty.Optional[str] = None,
-        domain_name: ty.Optional[str] = None,
-        project_id: ty.Optional[str] = None,
-        project_name: ty.Optional[str] = None,
-        project_domain_id: ty.Optional[str] = None,
-        project_domain_name: ty.Optional[str] = None,
+        trust_id: str | None = None,
+        system_scope: str | None = None,
+        domain_id: str | None = None,
+        domain_name: str | None = None,
+        project_id: str | None = None,
+        project_name: str | None = None,
+        project_domain_id: str | None = None,
+        project_domain_name: str | None = None,
         reauthenticate: bool = True,
         include_catalog: bool = True,
     ):
@@ -720,13 +712,12 @@ class OidcAccessToken(_OidcBase):
 
     def get_payload(
         self, session: ks_session.Session
-    ) -> ty.Dict[str, ty.Optional[str]]:
+    ) -> dict[str, str | None]:
         """OidcAccessToken does not require a payload."""  # noqa: D403
         return {}
 
-    # TODO(stephenfin): Deprecate and remove unused kwargs
     def get_unscoped_auth_ref(
-        self, session: ks_session.Session, **kwargs: ty.Any
+        self, session: ks_session.Session
     ) -> access.AccessInfoV3:
         """Authenticate with OpenID Connect and get back claims.
 
@@ -761,23 +752,23 @@ class OidcDeviceAuthorization(_OidcBase):
         identity_provider: str,
         protocol: str,
         client_id: str,
-        client_secret: ty.Optional[str],
-        # access_token_type intentionally skipped
+        client_secret: str | None = None,
+        access_token_type: str = "access_token",  # nosec B107
         scope: str = 'openid profile',
-        access_token_endpoint: ty.Optional[str] = None,
-        discovery_endpoint: ty.Optional[str] = None,
-        device_authorization_endpoint: ty.Optional[str] = None,
-        code_challenge: ty.Optional[str] = None,
-        code_challenge_method: ty.Optional[str] = None,
+        access_token_endpoint: str | None = None,
+        discovery_endpoint: str | None = None,
+        device_authorization_endpoint: str | None = None,
+        code_challenge: str | None = None,
+        code_challenge_method: str | None = None,
         *,
-        trust_id: ty.Optional[str] = None,
-        system_scope: ty.Optional[str] = None,
-        domain_id: ty.Optional[str] = None,
-        domain_name: ty.Optional[str] = None,
-        project_id: ty.Optional[str] = None,
-        project_name: ty.Optional[str] = None,
-        project_domain_id: ty.Optional[str] = None,
-        project_domain_name: ty.Optional[str] = None,
+        trust_id: str | None = None,
+        system_scope: str | None = None,
+        domain_id: str | None = None,
+        domain_name: str | None = None,
+        project_id: str | None = None,
+        project_name: str | None = None,
+        project_domain_id: str | None = None,
+        project_domain_name: str | None = None,
         reauthenticate: bool = True,
         include_catalog: bool = True,
     ):
@@ -794,8 +785,6 @@ class OidcDeviceAuthorization(_OidcBase):
         :param code_challenge_method: PKCE Challenge Method (RFC 7636).
         :type code_challenge_method: string
         """
-        # RFC 8628 only allows to retrieve an access_token
-        self.access_token_type = 'access_token'  # nosec B105
         self.device_authorization_endpoint = device_authorization_endpoint
         self.code_challenge_method = code_challenge_method
 
@@ -805,7 +794,7 @@ class OidcDeviceAuthorization(_OidcBase):
             protocol=protocol,
             client_id=client_id,
             client_secret=client_secret,
-            access_token_type=self.access_token_type,
+            access_token_type=access_token_type,
             scope=scope,
             access_token_endpoint=access_token_endpoint,
             discovery_endpoint=discovery_endpoint,
@@ -823,7 +812,7 @@ class OidcDeviceAuthorization(_OidcBase):
 
     def _get_device_authorization_endpoint(
         self, session: ks_session.Session
-    ) -> ty.Optional[str]:
+    ) -> str | None:
         """Get the endpoint for the OAuth 2.0 Device Authorization flow.
 
         This method will return the correct device authorization endpoint to
@@ -856,7 +845,7 @@ class OidcDeviceAuthorization(_OidcBase):
         code_verifier = _rand_b64.rstrip('=')  # strip padding as RFC says
         return code_verifier
 
-    def _generate_pkce_challenge(self) -> ty.Optional[str]:
+    def _generate_pkce_challenge(self) -> str | None:
         """Generate PKCE challenge string as defined in RFC 7636."""
         if self.code_challenge_method not in ('plain', 'S256'):
             raise exceptions.OidcInvalidCodeChallengeMethod()
@@ -873,7 +862,7 @@ class OidcDeviceAuthorization(_OidcBase):
 
     def get_payload(
         self, session: ks_session.Session
-    ) -> ty.Dict[str, ty.Optional[str]]:
+    ) -> dict[str, str | None]:
         """Get an authorization grant for the "device_code" grant type.
 
         :param session: a session object to send out HTTP requests.
@@ -882,7 +871,7 @@ class OidcDeviceAuthorization(_OidcBase):
         :returns: a python dictionary containing the payload to be exchanged
         :rtype: dict
         """
-        payload: ty.Dict[str, ty.Optional[str]]
+        payload: dict[str, str | None]
         device_authz_endpoint = self._get_device_authorization_endpoint(
             session
         )
@@ -892,10 +881,13 @@ class OidcDeviceAuthorization(_OidcBase):
 
         if self.client_secret:
             client_auth = (self.client_id, self.client_secret)
-            payload = {}
         else:
             client_auth = None
-            payload = {'client_id': self.client_id}
+
+        # rfc8628 does not require client_id when a client_secret is provided,
+        # but Microsoft EntraID does; these should be ignored by other IDPs
+        # when a client secret is provided.
+        payload = {'client_id': self.client_id, 'scope': self.scope}
 
         if self.code_challenge_method:
             self.code_challenge = self._generate_pkce_challenge()
@@ -935,9 +927,9 @@ class OidcDeviceAuthorization(_OidcBase):
         self.interval = int(op_response.json()["interval"])
         self.user_code = op_response.json()["user_code"]
         self.verification_uri = op_response.json()["verification_uri"]
-        self.verification_uri_complete = op_response.json()[
+        self.verification_uri_complete = op_response.json().get(
             "verification_uri_complete"
-        ]
+        )
 
         payload = {'device_code': self.device_code}
         if self.code_challenge_method:
@@ -945,9 +937,7 @@ class OidcDeviceAuthorization(_OidcBase):
         return payload
 
     def _get_access_token(
-        self,
-        session: ks_session.Session,
-        payload: ty.Dict[str, ty.Optional[str]],
+        self, session: ks_session.Session, payload: dict[str, str | None]
     ) -> str:
         """Poll token endpoint for an access token.
 
@@ -963,16 +953,26 @@ class OidcDeviceAuthorization(_OidcBase):
                 }
         :type payload: dict
         """
-        _logger.warning(
-            f"To authenticate please go to: "
-            f"{self.verification_uri_complete}"
-        )
+        # verification_uri_complete is optional and not implemented by EntraID
+        if self.verification_uri_complete:
+            _logger.warning(
+                f"To authenticate please go to: {self.verification_uri_complete}"
+            )
+        else:
+            _logger.warning(
+                f"To authenticate please go to {self.verification_uri} "
+                f"and enter the code {self.user_code}"
+            )
 
         if self.client_secret:
             client_auth = (self.client_id, self.client_secret)
         else:
             client_auth = None
-            payload.setdefault('client_id', self.client_id)
+
+        # rfc8628 does not require client_id when a client_secret is provided,
+        # but Microsoft EntraID does; this should be ignored by other IDPs
+        # when a client secret is provided.
+        payload.setdefault('client_id', self.client_id)
 
         access_token_endpoint = self._get_access_token_endpoint(session)
         encoded_payload = urlparse.urlencode(payload)
@@ -1003,7 +1003,9 @@ class OidcDeviceAuthorization(_OidcBase):
                         sanitized_response,
                     )
             except exceptions.http.BadRequest as exc:
-                error = exc.response and exc.response.json().get("error")
+                if exc.response is None:
+                    raise
+                error = exc.response.json().get("error")
                 if error != "authorization_pending":
                     raise
                 time.sleep(self.interval)
