@@ -51,7 +51,7 @@ class BaseIdentityLoader(base.BaseLoader[PluginT]):
     :py:class:`keystoneauth1.identity.BaseIdentityPlugin` class.
     """
 
-    def get_options(self) -> ty.List[opts.Opt]:
+    def get_options(self) -> list[opts.Opt]:
         options = super().get_options()
 
         options.extend(
@@ -69,7 +69,7 @@ class BaseV2Loader(BaseIdentityLoader[V2PluginT]):
     :py:class:`keystoneauth1.identity.v2.Auth` class.
     """
 
-    def get_options(self) -> ty.List[opts.Opt]:
+    def get_options(self) -> list[opts.Opt]:
         options = super().get_options()
 
         options.extend(
@@ -93,7 +93,7 @@ class BaseV3Loader(BaseIdentityLoader[V3PluginT]):
     :py:class:`keystoneauth1.identity.v3.Auth` class.
     """
 
-    def get_options(self) -> ty.List[opts.Opt]:
+    def get_options(self) -> list[opts.Opt]:
         options = super().get_options()
 
         options.extend(
@@ -142,7 +142,7 @@ class BaseFederationLoader(BaseV3Loader[V3FederationPluginT]):
     :py:class:`keystoneauth1.identity.v3.FederationBaseAuth` class.
     """
 
-    def get_options(self) -> ty.List[opts.Opt]:
+    def get_options(self) -> list[opts.Opt]:
         options = super().get_options()
 
         options.extend(
@@ -169,10 +169,11 @@ class BaseGenericLoader(BaseIdentityLoader[GenericPluginT]):
     This class defines options and handling that should be common to generic
     plugins. These plugins target the OpenStack identity service however are
     designed to be independent of API version. It provides the options expected
-    by the :py:class:`keystoneauth1.identity.v3.BaseGenericPlugin` class.
+    by the :py:class:`keystoneauth1.identity.generic.base.BaseGenericPlugin`
+    class.
     """
 
-    def get_options(self) -> ty.List[opts.Opt]:
+    def get_options(self) -> list[opts.Opt]:
         options = super().get_options()
 
         options.extend(
@@ -202,17 +203,21 @@ class BaseGenericLoader(BaseIdentityLoader[GenericPluginT]):
                 ),
                 opts.Opt(
                     'default-domain-id',
-                    help='Optional domain ID to use with v3 and v2 '
-                    'parameters. It will be used for both the user '
-                    'and project domain in v3 and ignored in '
-                    'v2 authentication.',
+                    help=(
+                        'Optional domain ID to use with v3 and v2 '
+                        'parameters. It will be used for both the user '
+                        'and project domain in v3 and ignored in '
+                        'v2 authentication.'
+                    ),
                 ),
                 opts.Opt(
                     'default-domain-name',
-                    help='Optional domain name to use with v3 API and v2 '
-                    'parameters. It will be used for both the user '
-                    'and project domain in v3 and ignored in '
-                    'v2 authentication.',
+                    help=(
+                        'Optional domain name to use with v3 API and v2 '
+                        'parameters. It will be used for both the user '
+                        'and project domain in v3 and ignored in '
+                        'v2 authentication.'
+                    ),
                 ),
             ]
         )
