@@ -33,10 +33,10 @@ class ServiceTokenAuthWrapper(plugin.BaseAuthPlugin):
         self.service_auth = service_auth
 
     def get_headers(
-        self, session: 'ks_session.Session', **kwargs: ty.Any
-    ) -> ty.Optional[ty.Dict[str, str]]:
-        headers = self.user_auth.get_headers(session, **kwargs) or {}
-        token = self.service_auth.get_token(session, **kwargs)
+        self, session: 'ks_session.Session'
+    ) -> dict[str, str] | None:
+        headers = self.user_auth.get_headers(session) or {}
+        token = self.service_auth.get_token(session)
         if token:
             headers[SERVICE_AUTH_HEADER_NAME] = token
 
@@ -51,48 +51,42 @@ class ServiceTokenAuthWrapper(plugin.BaseAuthPlugin):
         return user or service
 
     def get_connection_params(
-        self, session: 'ks_session.Session', **kwargs: ty.Any
-    ) -> ty.Dict[str, ty.Any]:
+        self, session: 'ks_session.Session'
+    ) -> plugin.ConnectionParams:
         # NOTE(jamielennox): This is also a bit of a guess but unlikely to be a
         # problem in practice. We don't know how merging connection parameters
         # between these plugins will conflict - but there aren't many plugins
         # that set this anyway.
         # Take the service auth params first so that user auth params will be
         # given priority.
-        params = self.service_auth.get_connection_params(session, **kwargs)
-        params.update(self.user_auth.get_connection_params(session, **kwargs))
+        params = self.service_auth.get_connection_params(session)
+        params.update(self.user_auth.get_connection_params(session))
         return params
 
     # TODO(jamielennox): Everything below here is a generic wrapper that could
     # be extracted into a base wrapper class. We can do this as soon as there
     # is a need for it, but we may never actually need it.
 
-    def get_token(
-        self, session: 'ks_session.Session', **kwargs: ty.Any
-    ) -> ty.Optional[str]:
-        return self.user_auth.get_token(session, **kwargs)
+    def get_token(self, session: 'ks_session.Session') -> str | None:
+        return self.user_auth.get_token(session)
 
     def get_endpoint(
         self, session: 'ks_session.Session', **kwargs: ty.Any
-    ) -> ty.Optional[str]:
+    ) -> str | None:
         return self.user_auth.get_endpoint(session, **kwargs)
 
-    def get_user_id(
-        self, session: 'ks_session.Session', **kwargs: ty.Any
-    ) -> ty.Optional[str]:
-        return self.user_auth.get_user_id(session, **kwargs)
+    def get_user_id(self, session: 'ks_session.Session') -> str | None:
+        return self.user_auth.get_user_id(session)
 
-    def get_project_id(
-        self, session: 'ks_session.Session', **kwargs: ty.Any
-    ) -> ty.Optional[str]:
-        return self.user_auth.get_project_id(session, **kwargs)
+    def get_project_id(self, session: 'ks_session.Session') -> str | None:
+        return self.user_auth.get_project_id(session)
 
     def get_sp_auth_url(
-        self, session: 'ks_session.Session', sp_id: str, **kwargs: ty.Any
-    ) -> ty.Optional[str]:
-        return self.user_auth.get_sp_auth_url(session, sp_id, **kwargs)
+        self, session: 'ks_session.Session', sp_id: str
+    ) -> str | None:
+        return self.user_auth.get_sp_auth_url(session, sp_id)
 
     def get_sp_url(
-        self, session: 'ks_session.Session', sp_id: str, **kwargs: ty.Any
-    ) -> ty.Optional[str]:
-        return self.user_auth.get_sp_url(session, sp_id, **kwargs)
+        self, session: 'ks_session.Session', sp_id: str
+    ) -> str | None:
+        return self.user_auth.get_sp_url(session, sp_id)
