@@ -13,17 +13,16 @@
 import json as jsonutils
 import logging
 import time
+import urllib.parse
 import uuid
 
 import fixtures
 import requests
 from requests_mock.contrib import fixture
-from six.moves.urllib import parse as urlparse
 import testtools
 
 
 class TestCase(testtools.TestCase):
-
     TEST_DOMAIN_ID = uuid.uuid4().hex
     TEST_DOMAIN_NAME = uuid.uuid4().hex
     TEST_GROUP_ID = uuid.uuid4().hex
@@ -39,7 +38,7 @@ class TestCase(testtools.TestCase):
     TEST_ROOT_URL = 'http://127.0.0.1:5000/'
 
     def setUp(self):
-        super(TestCase, self).setUp()
+        super().setUp()
         self.logger = self.useFixture(fixtures.FakeLogger(level=logging.DEBUG))
 
         fixtures.MockPatchObject(time, 'time', lambda: 1234)
@@ -80,9 +79,11 @@ class TestCase(testtools.TestCase):
 
         The qs parameter should be of the format \'foo=bar&abc=xyz\'
         """
-        expected = urlparse.parse_qs(qs, keep_blank_values=True)
-        parts = urlparse.urlparse(self.requests_mock.last_request.url)
-        querystring = urlparse.parse_qs(parts.query, keep_blank_values=True)
+        expected = urllib.parse.parse_qs(qs, keep_blank_values=True)
+        parts = urllib.parse.urlparse(self.requests_mock.last_request.url)
+        querystring = urllib.parse.parse_qs(
+            parts.query, keep_blank_values=True
+        )
         self.assertEqual(expected, querystring)
 
     def assertQueryStringContains(self, **kwargs):
@@ -95,8 +96,8 @@ class TestCase(testtools.TestCase):
         verified is that the parameter is present.
 
         """
-        parts = urlparse.urlparse(self.requests_mock.last_request.url)
-        qs = urlparse.parse_qs(parts.query, keep_blank_values=True)
+        parts = urllib.parse.urlparse(self.requests_mock.last_request.url)
+        qs = urllib.parse.parse_qs(parts.query, keep_blank_values=True)
 
         for k, v in kwargs.items():
             self.assertIn(k, qs)
@@ -127,7 +128,7 @@ class TestResponse(requests.Response):
 
     def __init__(self, data):
         self._text = None
-        super(TestResponse, self).__init__()
+        super().__init__()
         if isinstance(data, dict):
             self.status_code = data.get('status_code', 200)
             headers = data.get('headers')
@@ -143,12 +144,6 @@ class TestResponse(requests.Response):
         """Define equiality behavior of request and response."""
         return self.__dict__ == other.__dict__
 
-    # NOTE: This function is only needed by Python 2. If we get to point where
-    # we don't support Python 2 anymore, this function should be removed.
-    def __ne__(self, other):
-        """Define inequiality behavior of request and response."""
-        return not self.__eq__(other)
-
     @property
     def text(self):
         return self.content
